<?php
namespace Teknisa\FIN\Controllers;

use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\Libs\Service\CRUD as CrudService;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\FIN\Services\Ger38200ImportacaoContasPagarService;
use Teknisa\FIN\Modules\ContaPagar\Controllers\TitulosAPagarController;

class Ger38200ImportacaoContasPagarController extends TeknisaCrud{
    
    protected $titulosAPagarController;
    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param Ger38200ImportacaoContasPagarController $ger38200ImportacaoContasPagarService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment,
        Ger38200ImportacaoContasPagarService $ger38200ImportacaoContasPagarService,  TitulosAPagarController $titulosAPagarController){
            
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->ger38200ImportacaoContasPagarService = $ger38200ImportacaoContasPagarService;
        $this->nrorg = $environment->getNrOrgTrab();
        $this->titulosAPagarController = $titulosAPagarController;
    }

    /**
     * Importação de Contas a Pagar - Web Service
     * @param Row $row
     * @param Response $response
     */
    public function importacaoContasPagar(Data $dataset, Response $response){
        try {
            $rows = $dataset->getDataSet()->getRows();
            $dados = $rows[0];
            $dados["NRORG"] = $this->nrorg; // Inserido NRORG para ser usado no update da PARAMGERAL por garantia
            
            $dadosIntegracao = $this->ger38200ImportacaoContasPagarService->importacaoContasPagar($dados);
            $response->addDataSet(new DataSet('importacaoContasPagar', ['success' => 'S', 'dadosHCM' => $dadosIntegracao]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Geração de Contas a Pagar 
     * @param DataSet 
     * @param Response $response
     */
    public function geraTituloApagarImportacao(Data $dataset, Response $response){
        try {
            //Recebe os dados que foram enviados pela tela
            $rows = $dataset->getDataSet()->getRows();
            
            //Setando dados fixos, NRORG e CDOPERADOR 
            $rows[0]["NRORG"]        = $this->nrorg;
            $rows[0]["CDOPERADOR"]   = $this->environment->getCdOperador($rows);
            
            //Valida os erros de dados do pagamento Filial,Centro de Custo, Nota Fiscal, Cod.Fornecedor
            $erros  = $this->ger38200ImportacaoContasPagarService->validations($rows);
            
            if($erros){
                $response->addDataSet(new DataSet('importTitulos', array('failure' => true,'ERRO' => $erros)));
            } else {
                //Envia para o build para preparar o dataset contendo os dados necessários para inserir o título
                $dataBuild = $this->ger38200ImportacaoContasPagarService->buildTitulosAPagarDataSet($rows);
                
                //Envia para o build para preparar o dataset contendo os dados de rateio 
                $rateioBuild = $this->ger38200ImportacaoContasPagarService->buildRateioTitulosAPagar($rows);
                //Envia os dados já montados para inserir o contas a pagar(Rotina já existente);
                $this->ger38200ImportacaoContasPagarService->inserirContasAPagar($dataBuild,$response, $rateioBuild);
                $response->addDataSet(new DataSet('importTitulos', array('success' => true)));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}