<?php

namespace Teknisa\FIN\Controllers;

use Teknisa\FIN\Services\ImportacaoTitulosService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error; 
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\FIN\Modules\ContaPagar\Controllers\TitulosAPagarController;
use Teknisa\FIN\Modules\ContaReceber\Controllers\TitulosAReceberController;

use Teknisa\FIN\Controllers\TeknisaCrud;

class ImportacaoTitulosController extends TeknisaCrud {
    
    const TITULOS_PAGAR = 'P';
    const TITULOS_RECEBER = 'R';
    
    protected $service;
    protected $titulosAPagarController;
    protected $titulosAReceberController;
    protected $nrorg;
    protected $environment;
    
    public function __construct($crudService, ParameterBag $parameterBag, ImportacaoTitulosService $service,
        TitulosAPagarController $titulosAPagarController, TitulosAReceberController $titulosAReceberController) {
        $this->service = $service;
        $this->titulosAPagarController = $titulosAPagarController;
        $this->titulosAReceberController = $titulosAReceberController;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function importTituloPagar(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            
            //Lê a tabela de títulos da planilha.
            $table = $this->service->readTitulosTable($row, self::TITULOS_PAGAR);
            
            //Formata dados das linhas
            $table->setRows($this->service->formatTitulos($table, self::TITULOS_PAGAR));
            
            //Valida os dados da tabela.
            $dbColumnValues = array('CDEMPRESA' => $row['CDEMPRESA']);
            $this->service->validateTitulosTable($table, self::TITULOS_PAGAR, $dbColumnValues);
            
            $rows = $table->getRows();
            $dataSet = $this->service->buildTitulosAPagarDataSet($row['CDEMPRESA'], $rows);
            
            $this->titulosAPagarController->geraTitulosAPagar($dataSet, $response);
            $response->addDataSet(new DataSet('importTitulos', array('success' => true)));
        }
        catch(Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function importTituloReceber(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            
            //Lê a tabela de títulos da planilha.
            $table = $this->service->readTitulosTable($row, self::TITULOS_RECEBER);
            
            //Formata dados das linhas
            $table->setRows($this->service->formatTitulos($table, self::TITULOS_RECEBER));
            
            //Valida os dados da tabela.
            $dbColumnValues = array('CDEMPRESA' => $row['CDEMPRESA']);
            $this->service->validateTitulosTable($table, self::TITULOS_RECEBER, $dbColumnValues);
        
            $rows = $table->getRows();
            $dataSet = $this->service->buildTitulosAReceberDataSet($row['CDEMPRESA'], $rows);
            
            $this->titulosAReceberController->geraTitulosAReceber($dataSet, $response);
            
            $response->addDataSet(new DataSet('importTitulos', array('success' => true)));
        }
        catch(Exception $e){
            $errors = $e->getErrors();
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error($e->getMessage(), 500));
        }
        catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
}