<?php

namespace Teknisa\FIN\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Financeiro\Entities\GeralRec;

class AutomacaoFinanceiraFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->connection    = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
    }
    
    public function conectar($nrorg, $useVpd, $newEntityManager, $newFileUploader){
        if($useVpd){
            $this->entityManager = $newEntityManager;
            $this->connection = $newEntityManager->getConnection();

        }
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
	    $this->environment->setCurrentUserId('AUTOMATICO');
    }

    /**
     * Busca Parametrização Títulos a Receber
     * @param $nrorg
     * @return $geralrec
     */
    public function getGeralrec($nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;

        /** @var GeralRec $geralrec */
        $geralrec = $this->entityManager->getRepository(GeralRec::class)->findOneBy($params);

        return $geralrec;
    }


    /**
     * Busca Boletos para Envio Emissao
     * @param $nrorg
     * @param $vencimento
     * @return array
     */
    public function getBoletosParaEnvioEmissao(
        $nrorg, $vencimento
    ){
        $params = array();

        $params['P_NRORG']       = $nrorg;
        $params['P_DTVENBOLREC'] = $vencimento;
        
        $sql = "SELECT DISTINCT *
                  FROM BOLETRECEBER BR, ENDECLIE EC, CONTCORR CC, CONTARECEBER CR, NOTAFISCAL NF, ENDEFILI EF
                 WHERE BR.CDREDUCONT = CC.CDREDUCONT
                   AND BR.CDCLIENTE = EC.CDCLIENTE
                   AND BR.CDFILIAL = CR.CDFILIBOLREC
                   AND BR.NRLANBOLREC = CR.NRLANBOLREC
                   AND BR.CDCLIENTE = EC.CDCLIENTE
                   AND CR.CDFILIAL = NF.CDFILIAL
                   AND CR.NRLANCTONF = NF.NRLANCTONF
                   AND NF.CDFILIEMIT = EF.CDFILIAL  
                   AND EF.IDTPENDEFILI = 'C'
                   AND EC.IDTPENDECLIE = 'C'
                   AND BR.IDSTATBREC = 'E'
                   AND BR.IDSITUCNABR <> '00'
                   AND BR.NRORG       = :P_NRORG
                   AND BR.DTVENBOLREC = :P_DTVENBOLREC";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    /**
     * Busca Boletos para Envio Cobranca
     * @param $nrorg
     * @param $vencimento
     * @return array
     */
    public function getBoletosParaEnvioCobranca(
        $nrorg, $vencimento
    ){
        $params = array();

        $params['P_NRORG']       = $nrorg;
        $params['P_DTVENBOLREC'] = $vencimento;
        
        $sql = "SELECT DISTINCT *
                  FROM BOLETRECEBER BR, ENDECLIE EC, CONTCORR CC, CONTARECEBER CR, NOTAFISCAL NF, ENDEFILI EF
                 WHERE BR.CDREDUCONT = CC.CDREDUCONT
                   AND BR.CDCLIENTE = EC.CDCLIENTE
                   AND BR.CDFILIAL = CR.CDFILIBOLREC
                   AND BR.NRLANBOLREC = CR.NRLANBOLREC
                   AND BR.CDCLIENTE = EC.CDCLIENTE
                   AND CR.CDFILIAL = NF.CDFILIAL
                   AND CR.NRLANCTONF = NF.NRLANCTONF
                   AND NF.CDFILIEMIT = EF.CDFILIAL  
                   AND EF.IDTPENDEFILI = 'C'
                   AND EC.IDTPENDECLIE = 'C'
                   AND BR.IDSTATBREC = 'E'
                   AND BR.IDSITUCNABR <> '00'
                   AND BR.NRORG       = :P_NRORG
                   AND BR.DTVENBOLREC = :P_DTVENBOLREC";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
}