<?php

namespace Teknisa\FIN\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;

class RecalculoContabilizacaoTitulosBaixasFactory {

    protected $connection;
    protected $nrOrg;

    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->connection = $entityManager->getConnection();
        $this->nrOrg = $environment->getNrOrg();
    }

    public function findTitulosPagar(array $params) {

        $params["NRORG"] = $this->nrOrg;

        $sql = "SELECT CP.NRORG,
            	NF.NRNOTAFISC || '/' || NF.CDSERIEENTR AS DOCUMENTO,
            	CP.DSCONTAPAG,
            	CP.CDFILIAL,
            	CP.NRLANCTONF,
            	CP.NRSEQVENPAG,
            	CP.DTATUAVENPAG,
            	CP.NRPARCELAPAG,
            	CP.CDMOTIPROR,
            	CP.CDMOTIPARC,
            	CP.IDSTATPAG,
            	CP.CDTPCTPAGAR,
            	CP.VRATUAPAG,
            	CP.CDEMPRESA,
            	FO.NMRAZSOCFORN,
            	FO.NRINSJURFORN,
            	FO.CDFORNECED,
            	FO.NMFANTFORN,
            	TP.DSTPCTPAGAR,
            	NF.CDSERIE,
            	NF.CDFILIEMIT,
            	NF.NRNOTAFISC,
            	NF.NRLANCTBDOC,
            	NF.IDSTATUS,
            	NF.VRNOTAFISC,
            	NF.DTENTRSAID,
            	NF.CDFILIRECE,
            	NF.CDSERIEENTR,
            	TP.IDCTBLCTPAG,
            	TP.IDUTLANCCPAG,
            	PF.NRTAMNUMNFUT
            FROM FILIAL FI,
            	TIPOCTPAGAR TP,
            	TIPOOPERAC OP,
            	FORNECEDOR FO,
            	NOTAFISCAL NF,
            	CONTAPAGAR CP,
            	PARAMFILIAL PF
            WHERE (CP.CDEMPRESA = :P_CDEMPRESA)
                AND (CP.NRORG   = :NRORG)
            	AND (NF.CDFILIAL = CP.CDFILIAL)
            	AND (NF.NRLANCTONF = CP.NRLANCTONF)
            	AND (FI.CDFILIAL = NF.CDFILIRECE)
            	AND (OP.CDTIPOOPER = NF.CDTIPOOPER)
            	AND (FO.CDFORNECED = NF.CDFORNECED)
            	AND (TP.CDTPCTPAGAR = CP.CDTPCTPAGAR)
            	AND (CP.CDFILIAL = PF.CDFILIAL)
            	AND (OP.IDENTRSAIDOP = 'E')
            	AND (NF.IDSTATUS IN ('A', 'E'))
            	AND (NF.IDSISTEMA = 'PAG')";
        
        $forneced = " AND (NF.CDFORNECED IN (";
        
        if(isset($params['P_CDFORNECED'][0])) {
            $sqlFornced = $this->getSqlSelectNative($forneced, $params['P_CDFORNECED']);
            $sql.=$sqlFornced;
            unset($params["P_CDFORNECED"]);
        }
 
        $sql2=" AND (
            		(LPAD(NF.NRNOTAFISC, NVL(PF.NRTAMNUMNFUT, 7), '0') BETWEEN LPAD(NVL(:P_NRNOTAFISCINI, '0'), NVL(PF.NRTAMNUMNFUT, 7), '0') AND LPAD(NVL(:P_NRNOTAFISCFIN, '99999999999999999999'), NVL(PF.NRTAMNUMNFUT, 7), '0'))
            		OR (:P_NRNOTAFISCINI IS NULL AND :P_NRNOTAFISCFIN IS NULL)
            		)
            	AND (
            		(NF.DTEMISSAO BETWEEN NVL(:P_DTEMISSAOINI, SYSDATE) AND NVL(:P_DTEMISSAOFIN, SYSDATE))
            		OR (:P_DTEMISSAOINI IS NULL AND :P_DTEMISSAOFIN IS NULL)
            		)
            	AND (
            		(NF.DTENTRSAID BETWEEN NVL(:P_DTENTRADAINI, SYSDATE) AND NVL(:P_DTENTRADAFIN, SYSDATE))
            		OR (:P_DTENTRADAINI IS NULL AND :P_DTENTRADAFIN IS NULL)
            		)
            	AND (
            		INSTR(:P_CDFILIORIG, '_'||NF.CDFILIRECE||'_') > 0
            		OR :P_CDFILIORIG IS NULL
            		)
            	AND (
            		INSTR(:P_CDFILILANC, '_'||NF.CDFILIAL||'_') > 0
            		OR :P_CDFILILANC IS NULL
            		)
            	AND (
            		(NF.VRNOTAFISC BETWEEN :P_VRDOCUINI AND :P_VRDOCUFIN)
            		OR (:P_VRDOCUINI IS NULL AND :P_VRDOCUFIN IS NULL)
            		)
            	AND (
            		(CP.DTATUAVENPAG BETWEEN NVL(:P_DTVENCINI, SYSDATE) AND NVL(:P_DTVENCFIN, SYSDATE))
            		OR (:P_DTVENCINI IS NULL AND :P_DTVENCFIN IS NULL)
            		)
            	AND (
            		INSTR(:P_CDTPCTPAGAR, '_'||CP.CDTPCTPAGAR||'_') > 0
            		OR :P_CDTPCTPAGAR IS NULL
            		)
            	AND (
            		(CP.VRORIGPAG BETWEEN TRUNC(:P_VRORIGINI, 2) AND TRUNC(:P_VRORIGFIN, 2))
            		OR (:P_VRORIGINI IS NULL AND :P_VRORIGFIN IS NULL)
            		)
            	AND (
            		(CP.VRATUAPAG BETWEEN :P_VRLIQUINI AND :P_VRLIQUFIN)
            		OR (:P_VRLIQUINI IS NULL AND :P_VRLIQUFIN IS NULL)
            		)
            	AND (CP.IDSTATPAG IN ('A', 'B'))";

        $sql.=$sql2;
        $this->completeParamsArrayAccordingToQuery($sql, $params);
        if(array_key_exists("P_IDCONTABILIZA", $params)){
            if ($params["P_IDCONTABILIZA"] == 'S') {
                $sql.= " AND (NRLANCTBDOC IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = NRLANCTBDOC))";
            } else if ($params["P_IDCONTABILIZA"] == 'N') {
                $sql.= " AND (NRLANCTBDOC NOT IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = NRLANCTBDOC))";
            }
        }
        if(array_key_exists("P_IDVENCIDAS", $params)){
            if ($params["P_IDVENCIDAS"] == 'S') {
                $sql.= " AND CP.DTATUAVENPAG  < TRUNC(SYSDATE)";
            } else if ($params["P_IDVENCIDAS"] == 'N') {
                $sql.= " AND CP.DTATUAVENPAG  >= TRUNC(SYSDATE)";
            }
        }
        unset($params["P_IDCONTABILIZA"]);
        unset($params["P_IDVENCIDAS"]);
     
        $sql.= " ORDER BY CP.DTATUAVENPAG,
            	FO.NMFANTFORN";

        $std = $this->connection->prepare($sql);
    
        $std->execute($params);

        return $std->fetchAll();
    }

    public function findTitulosReceber($params) {

        $params["NRORG"] = $this->nrOrg;
        $sql = "SELECT
                    NF.NRNOTAFISC || '/' || NF.CDSERIE AS DOCUMENTO,
                	CR.DSCONTAREC,
                	CR.CDFILIAL,
                	CR.NRLANCTONF,
                	CR.NRSEQVENREC,
                	CR.DTATUAVENREC,
                	CR.NRPARCELAREC,
                	CR.CDMOTIPROR,
                	CR.CDMOTIPARC,
                	CR.IDSTATREC,
                	CR.CDTPCTRECEB,
                	CR.VRATUAREC,
                	CR.CDEMPRESA,
                	CL.NMRAZSOCCLIE,
                	CL.NRINSJURCLIE,
                	CL.CDCLIENTE,
                	CL.NMFANTCLIE,
                	TR.DSTPCTRECEB,
                	NF.CDSERIE,
                	NF.CDFILIEMIT,
                	NF.NRNOTAFISC,
                	NF.NRLANCTBDOC,
                	NF.IDSTATUS,
                	NF.VRNOTAFISC,
                	NF.DTENTRSAID,
                	TR.IDCTBLCTREC,
                	CL.IDTPIJURCLIE,
                	TR.IDUTLANCCREC,
                	PF.NRTAMNUMNFUT
                FROM NOTAFISCAL NF,
                	CLIENTE CL,
                	FILIAL FI,
                	TIPOCTRECEBER TR,
                	TIPOOPERAC OP,
                	CONTARECEBER CR,
                	PARAMFILIAL PF
                WHERE (CR.CDEMPRESA = :P_CDEMPRESA)
                    AND (CR.NRORG = :NRORG)
                	AND (NF.IDBLOQUEADO = 'S')
                	AND (NF.IDSTATUS IN ('A', 'E'))
                	AND (NF.CDFILIAL = CR.CDFILIAL)
                	AND (NF.NRLANCTONF = CR.NRLANCTONF)
                	AND (FI.CDFILIAL = NF.CDFILIEMIT)
                	AND (OP.IDENTRSAIDOP = 'S')
                	AND (OP.CDTIPOOPER = NF.CDTIPOOPER)
                	AND (CL.CDCLIENTE = NF.CDCLIENTE)
                	AND (CR.CDFILIAL = PF.CDFILIAL)
                	AND (TR.CDTPCTRECEB(+) = CR.CDTPCTRECEB)
                	AND (TR.IDCTBLCTREC = 'S')
                	AND ((LPAD(NF.NRNOTAFISC, NVL(PF.NRTAMNUMNFUT, 7), '0') BETWEEN LPAD(NVL(:P_NRNOTAFISCINI, '0'), NVL(PF.NRTAMNUMNFUT, 7), '0') AND LPAD(NVL(:P_NRNOTAFISCFIN, '99999999999999999999'), NVL(PF.NRTAMNUMNFUT, 7), '0'))
                	    OR (:P_NRNOTAFISCINI IS NULL AND :P_NRNOTAFISCFIN IS NULL))
                	AND ((NF.DTEMISSAO BETWEEN NVL(:P_DTEMISSAOINI, SYSDATE) AND NVL(:P_DTEMISSAOFIN, SYSDATE)) OR (:P_DTEMISSAOINI IS NULL AND :P_DTEMISSAOFIN IS NULL))
                	AND ((NF.DTENTRSAID BETWEEN NVL(:P_DTSAIDAINI, SYSDATE) AND NVL(:P_DTSAIDAFIN, SYSDATE)) OR (:P_DTSAIDAINI IS NULL AND :P_DTSAIDAFIN IS NULL))
                	AND (INSTR(:P_CDFILIORIG, '_'||NF.CDFILIEMIT||'_') > 0 OR :P_CDFILIORIG IS NULL)
                	AND (INSTR(:P_CDFILILANC, '_'||NF.CDFILIAL||'_') > 0 OR :P_CDFILILANC IS NULL)
                	AND ((ROUND(NF.VRNOTAFISC, 2) BETWEEN ROUND(:P_VRDOCUINI, 2) AND ROUND(:P_VRDOCUFIN, 2)) OR (:P_VRDOCUINI IS NULL AND :P_VRDOCUFIN IS NULL))
                	AND ((CR.DTATUAVENREC BETWEEN NVL(:P_DTVENCINI, SYSDATE) AND NVL(:P_DTVENCFIN, SYSDATE)) OR (:P_DTVENCINI IS NULL AND :P_DTVENCFIN IS NULL))
                	AND (INSTR(:P_CDTPCTRECEB, '_'||CR.CDTPCTRECEB||'_') > 0 OR :P_CDTPCTRECEB IS NULL)
                	AND ((ROUND(CR.VRORIGREC, 2) BETWEEN ROUND(:P_VRORIGINI, 2) AND ROUND(:P_VRORIGFIN, 2)) OR (:P_VRORIGINI IS NULL AND :P_VRORIGFIN IS NULL))
                	AND ((ROUND(CR.VRATUAREC, 2) BETWEEN ROUND(:P_VRLIQUINI, 2) AND ROUND(:P_VRLIQUFIN, 2)) OR (:P_VRLIQUINI IS NULL AND :P_VRLIQUFIN IS NULL))
                	AND (
                		(NF.IDSISTEMA = 'REC')
                		OR (NF.IDSISTEMA = 'PDV')
                		)";
        $cliente = " AND (NF.CDCLIENTE IN (";
        
        if(isset($params['P_CDCLIENTE'][0])) {
            $sqlCliente = $this->getSqlSelectNative($cliente, $params['P_CDCLIENTE']);
            $sql.=$sqlCliente;
            unset($params["P_CDCLIENTE"]);
        }
        
        $this->completeParamsArrayAccordingToQuery($sql, $params);
        if(array_key_exists("P_IDCONTABILIZA", $params)){
            if ($params["P_IDCONTABILIZA"] == 'S') {
                $sql.= " AND (NRLANCTBDOC IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = NRLANCTBDOC))";
            } else if ($params["P_IDCONTABILIZA"] == 'N') {
                $sql.= " AND (NRLANCTBDOC NOT IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = NRLANCTBDOC))";
            }
            unset($params["P_IDCONTABILIZA"]);
        }

        

        $sql.= " ORDER BY CR.DTATUAVENREC,
                	CL.NMRAZSOCCLIE";

        $std = $this->connection->prepare($sql);

        $std->execute($params);

        return $std->fetchAll();

    }

    public function findBaixasPagar(array $params) {
        $params["NRORG"] = $this->nrOrg;
        $sql = "SELECT DISTINCT BP.CDFILIAL,
                	BP.NRBAIXAPAG,
                	BP.CDEMPRESA,
                	BP.NRMOVBANC,
                	BP.CDREDUCONT,
                	BP.NRCHEQUE,
                	BP.DTBAIXAPAG,
                	BP.VRBAIXAPAG,
                	BP.DSNOMCHEQUE,
                	BP.IDSTEMICHEQ,
                	BP.NRLCTBBPAG,
                	BP.IDSTATUSBPAG,
                	BP.CDOPERADOR,
                	CC.CDBANCO,
                	CC.CDAGENCIA,
                	CC.CDCONTCORR,
                	CC.DSCONTCORR,
                	CC.CDCONTCTBL,
                	BC.CDPICTCONT,
                	BP.DSBAIXAPAG,
                	BP.CDTPBAIXAPAG,
                	TBP.IDBAIXAUTILP,
                	TBP.NMTPBAIXAPAG,
                	TBP.IDCTBLBXPAG,
                	BP.DTLANCBAIXAP,
                	OP.NMOPERADOR,
                	BC.CDPICTAGEN,
                	BO.CDFILIBAIPAG,
                	TBP.IDPARTIDABOLPAG,
                	PF.NRTAMNUMNFUT
                FROM OPERADOR OP,
                	TIPOBAIXAPAG TBP,
                	CONTCORR CC,
                	BANCO BC,
                	NOTAFISCAL NF,
                	CONTAPAGAR CP,
                	BOLETPAGAR BO,
                	BAIXAPAGA BP,
                	PARAMFILIAL PF
                WHERE (BP.CDEMPRESA = :P_CDEMPRESA)
                    AND (BP.NRORG = :NRORG)
                	AND (CC.CDREDUCONT = BP.CDREDUCONT)
                	AND (BO.CDFILIBAIPAG = BP.CDFILIAL)
                	AND (BO.NRBAIXAPAG = BP.NRBAIXAPAG)
                	AND (CP.CDFILIBPAG = BO.CDFILIAL)
                	AND (CP.NRLANBOLPAG = BO.NRLANBOLPAG)
                	AND (NF.CDFILIAL = CP.CDFILIAL)
                	AND (NF.NRLANCTONF = CP.NRLANCTONF)
                	AND (CC.CDBANCO = BC.CDBANCO)
                	AND (BP.CDTPBAIXAPAG = TBP.CDTPBAIXAPAG)
                	AND (BP.CDFILIAL = PF.CDFILIAL)
                	AND (BP.CDOPERADOR = OP.CDOPERADOR(+))
                	AND (NF.IDSTATUS IN ('A'))
                	AND (BP.IDSTATUSBPAG = 'E')
                	AND (TBP.IDCTBLBXPAG = 'S')
                	AND ((BP.DTBAIXAPAG BETWEEN NVL(:P_DTBAIXAPAGINI, SYSDATE) AND NVL(:P_DTBAIXAPAGFIN, SYSDATE))
                			OR (:P_DTBAIXAPAGINI IS NULL AND :P_DTBAIXAPAGFIN IS NULL))
                	AND ((BP.DTLANCBAIXAP BETWEEN NVL(:P_DTLANCBAIXAPINI, SYSDATE) AND NVL(:P_DTLANCBAIXAPFIN, SYSDATE))
                			OR (:P_DTLANCBAIXAPINI IS NULL AND :P_DTLANCBAIXAPFIN IS NULL))
                	AND (INSTR(:P_CDTPBAIXAPAG, '_'||BP.CDTPBAIXAPAG||'_') > 0
                			OR (:P_CDTPBAIXAPAG IS NULL))
                	AND ((ROUND(BP.VRBAIXAPAG, 2) BETWEEN ROUND(:P_VRBAIXAPAGINI, 2) AND ROUND(:P_VRBAIXAPAGFIN, 2))
                			OR (:P_VRBAIXAPAGINI IS NULL OR :P_VRBAIXAPAGFIN IS NULL))
                	AND (INSTR(:P_CDREDUCONT, '_'||BP.CDREDUCONT||'_') > 0
                			OR (:P_CDREDUCONT IS NULL))
                	AND (INSTR(:P_CDOPERADOR, '_'||BP.CDOPERADOR||'_') > 0
                			OR (:P_CDOPERADOR IS NULL))
                	AND ((CP.DTATUAVENPAG BETWEEN NVL(:P_DTVENCINI, SYSDATE) AND NVL(:P_DTVENCFIN, SYSDATE))
                			OR (:P_DTVENCINI IS NULL AND :P_DTVENCFIN IS NULL))
                	AND (INSTR(:P_CDTPCTPAGAR, '_'||CP.CDTPCTPAGAR||'_') > 0
                            OR :P_CDTPCTPAGAR IS NULL)
                	AND ((ROUND(CP.VRORIGPAG, 2) BETWEEN TRUNC(:P_VRORIGINI, 2) AND TRUNC(:P_VRORIGFIN, 2))
                            OR (:P_VRORIGINI IS NULL AND :P_VRORIGFIN IS NULL))
                	AND ((ROUND(CP.VRATUAPAG, 2) BETWEEN ROUND(:P_VRLIQUINI, 2) AND ROUND(:P_VRLIQUFIN, 2))
                    		OR (:P_VRLIQUINI IS NULL AND :P_VRLIQUFIN IS NULL))
                	AND ((LPAD(NF.NRNOTAFISC, NVL(PF.NRTAMNUMNFUT, 7), '0') BETWEEN LPAD(:P_NRNOTAFISCINI, NVL(PF.NRTAMNUMNFUT, 7), '0') AND LPAD(:P_NRNOTAFISCFIN, NVL(PF.NRTAMNUMNFUT, 7), '0'))
                    		OR (:P_NRNOTAFISCINI IS NULL OR :P_NRNOTAFISCFIN IS NULL))
                    AND (INSTR(:P_CDFILIORIG, '_'||NF.CDFILIRECE||'_') > 0
                    		OR :P_CDFILIORIG IS NULL)
                	AND (INSTR(:P_CDFILILANC, '_'||NF.CDFILIAL||'_') > 0
                    		OR :P_CDFILILANC IS NULL)";
        
        $forneced = " AND (NF.CDFORNECED IN (";
        
        if(isset($params['P_CDFORNECED'][0])) {
            $sqlFornced = $this->getSqlSelectNative($forneced, $params['P_CDFORNECED']);
            $sql.=$sqlFornced;
            unset($params["P_CDFORNECED"]);
        }
        
        $this->completeParamsArrayAccordingToQuery($sql, $params);
        if(array_key_exists("P_IDCONTABILIZA", $params)){
            if ($params["P_IDCONTABILIZA"] == 'S') {
                $sql.= " AND (BP.NRLCTBBPAG IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = BP.NRLCTBBPAG))";
            } else if ($params["P_IDCONTABILIZA"] == 'N') {
                $sql.= " AND (BP.NRLCTBBPAG NOT IN (SELECT NRLANCCONT
                        FROM LANCCONT LC
                        WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                              LC.NRLANCCONT = BP.NRLCTBBPAG))";
            }
            unset($params["P_IDCONTABILIZA"]);
        }

        $sql.= " ORDER BY BP.DTBAIXAPAG";

        $std = $this->connection->prepare($sql);

        $std->execute($params);

        return $std->fetchAll();
    }

    public function findBaixasReceber(array $params) {
        unset($params['P_SLNOSNUMREC']);
        $params["NRORG"] = $this->nrOrg;

        if (isset($params["P_CDNOSNUMREC"])) {
            $sql = "SELECT DISTINCT
                      BR.CDFILIAL,
                      BR.NRBAIXAREC,
                      BR.CDEMPRESA,
                      BR.NRMOVBANC,
                      BR.CDREDUCONT,
                      BR.DTBAIXAREC,
                      BR.VRBAIXAREC,
                      BR.NRLCTBBREC,
                      BR.IDSTATUSBREC,
                      CC.CDBANCO,
                      CC.CDAGENCIA,
                      CC.CDCONTCORR,
                      CC.DSCONTCORR,
                      CC.CDCONTCTBL,
                      BC.CDPICTCONT,
                      TB.CDTPBAIXAREC,
                      TB.NMTPBAIXAREC,
                      TB.IDCTBLBXREC,
                       BC.CDPICTAGEN,
                       DECODE((SELECT 1
                               FROM BOLETRECEBER B
                             WHERE NRMOVICRED IS NOT NULL
                               AND B.NRLANBOLREC = BO.NRLANBOLREC
                               AND B.CDFILIAL    = BO.CDFILIAL),1,'S','N') IDBAIXABOLDESC
                    FROM
                      CONTCORR      CC,
                      BANCO         BC,
                      BOLETRECEBER  BO,
                      BAIXARECEBIDA BR,
                      TIPOBAIXAREC  TB,
                      CONTARECEBER  CR,
                      NOTAFISCAL    NF
                    WHERE
                      (BR.NRORG        = :NRORG)          AND
                      (CR.CDFILIAL     = NF.CDFILIAL)     AND
                      (CR.NRLANCTONF   = NF.NRLANCTONF)   AND
                      (BO.CDFILIAL     = CR.CDFILIBOLREC) AND
                      (BO.NRLANBOLREC  = CR.NRLANBOLREC)  AND
                      (BR.CDEMPRESA    = :P_CDEMPRESA       ) AND
                      (BR.CDFILIAL     = BO.CDFILIBAIREC) AND
                      (BR.NRBAIXAREC   = BO.NRBAIXAREC  ) AND
                      (BR.CDREDUCONT   = CC.CDREDUCONT  ) AND
                      (CC.CDBANCO      = BC.CDBANCO     ) AND
                      (TB.CDTPBAIXAREC = BR.CDTPBAIXAREC) AND
                      (BR.IDSTATUSBREC = 'E') AND
                      (TB.IDCTBLBXREC = 'S')
                     AND BO.CDNOSNUMREC LIKE :P_CDNOSNUMREC";
                     if(array_key_exists("P_IDCONTABILIZA", $params)){
                         if ($params["P_IDCONTABILIZA"] == 'S') {
                            $sql.= " AND (BR.NRLCTBBREC IN (SELECT NRLANCCONT
                            FROM LANCCONT LC
                            WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                                  LC.NRLANCCONT = BR.NRLCTBBREC))";
                           
                        } else if ($params["P_IDCONTABILIZA"] == 'N') {
                            $sql.= " AND (BR.NRLCTBBREC NOT IN (SELECT NRLANCCONT
                                    FROM LANCCONT LC
                                    WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                                          LC.NRLANCCONT = BR.NRLCTBBREC))";
                            
                        }
                        unset($params["P_IDCONTABILIZA"]);
                     }
                 
        }
        else {

            $sql = "SELECT DISTINCT
                    	BR.CDFILIAL,
                    	BR.NRBAIXAREC,
                    	BR.CDEMPRESA,
                    	BR.NRMOVBANC,
                    	BR.CDREDUCONT,
                    	BR.DTBAIXAREC,
                    	BR.VRBAIXAREC,
                    	BR.NRLCTBBREC,
                    	BR.IDSTATUSBREC,
                    	CC.CDBANCO,
                    	CC.CDAGENCIA,
                    	CC.CDCONTCORR,
                    	CC.DSCONTCORR,
                    	CC.CDCONTCTBL,
                    	BC.CDPICTCONT,
                    	TB.CDTPBAIXAREC,
                    	TB.NMTPBAIXAREC,
                    	TB.IDCTBLBXREC,
                    	BC.CDPICTAGEN,
                    	PF.NRTAMNUMNFUT,
                    	DECODE((
                    			SELECT 1
                    			FROM BOLETRECEBER B
                    			WHERE NRMOVICRED IS NOT NULL
                    				AND B.NRLANBOLREC = BO.NRLANBOLREC
                    				AND B.CDFILIAL = BO.CDFILIAL
                    			), 1, 'S', 'N') IDBAIXABOLDESC
                    FROM CONTCORR CC,
                    	BANCO BC,
                    	BOLETRECEBER BO,
                    	BAIXARECEBIDA BR,
                    	TIPOBAIXAREC TB,
                    	CONTARECEBER CR,
                    	NOTAFISCAL NF,
                    	PARAMFILIAL PF
                    WHERE (BR.NRORG = :NRORG)
                        AND (CR.CDFILIAL = NF.CDFILIAL)
                    	AND (CR.NRLANCTONF = NF.NRLANCTONF)
                    	AND (BO.CDFILIAL = CR.CDFILIBOLREC)
                    	AND (BO.NRLANBOLREC = CR.NRLANBOLREC)
                    	AND (BR.CDEMPRESA = :P_CDEMPRESA)
                    	AND (BR.CDFILIAL = BO.CDFILIBAIREC)
                    	AND (BR.NRBAIXAREC = BO.NRBAIXAREC)
                    	AND (BR.CDREDUCONT = CC.CDREDUCONT)
                    	AND (CC.CDBANCO = BC.CDBANCO)
                    	AND (TB.CDTPBAIXAREC = BR.CDTPBAIXAREC)
                    	AND (BR.CDFILIAL = PF.CDFILIAL)
                    	AND (BR.IDSTATUSBREC = 'E')
                    	AND (TB.IDCTBLBXREC = 'S')
                    	AND ((ROUND(BR.VRBAIXAREC, 2) BETWEEN ROUND(:P_VRBAIXARECINI, 2) AND ROUND(:P_VRBAIXARECFIN, 2))
                			OR (:P_VRBAIXARECINI IS NULL OR :P_VRBAIXARECFIN IS NULL))
                    	AND ((LPAD(NF.NRNOTAFISC, NVL(PF.NRTAMNUMNFUT, 7), 0) BETWEEN LPAD(:P_NRNOTAFISCINI, NVL(PF.NRTAMNUMNFUT, 7), 0) AND LPAD(:P_NRNOTAFISCFIN, NVL(PF.NRTAMNUMNFUT, 7), 0))
                    		 OR (:P_NRNOTAFISCFIN IS NULL AND :P_NRNOTAFISCINI IS NULL))
                    	AND (INSTR(:P_CDFILIORIG, '_'||NF.CDFILIEMIT||'_') > 0 OR :P_CDFILIORIG IS NULL)
                    	AND (INSTR(:P_CDFILILANC, '_'||NF.CDFILIAL||'_') > 0 OR :P_CDFILILANC IS NULL)
                    	AND ((BR.DTBAIXAREC BETWEEN :P_DTBAIXARECINI AND :P_DTBAIXARECFIN)
                    		 OR (:P_DTBAIXARECINI IS NULL AND :P_DTBAIXARECFIN IS NULL))
                    	AND (INSTR(:P_CDTPBAIXAREC, '_'||BR.CDTPBAIXAREC||'_') > 0 OR :P_CDTPBAIXAREC IS NULL)
                    	AND (INSTR(:P_CDREDUCONT, '_'||BR.CDREDUCONT||'_') > 0 OR :P_CDREDUCONT IS NULL)
                    	AND ((CR.DTATUAVENREC BETWEEN :P_DTVENCINI AND :P_DTVENCFIN)
                    		 OR (:P_DTVENCINI IS NULL AND :P_DTVENCFIN IS NULL))
                    	AND (INSTR(:P_CDOPERADOR, '_'||BR.CDOPERADOR||'_') > 0 OR :P_CDOPERADOR IS NULL)
                    	AND (INSTR(:P_CDTPRECEB, '_'||CR.CDTPCTRECEB||'_') > 0 OR :P_CDTPRECEB IS NULL)
                    	AND ((ROUND(CR.VRORIGREC, 2) BETWEEN ROUND(:P_VRORIGINI, 2) AND ROUND(:P_VRORIGFIN, 2))
                    		 OR (:P_VRORIGINI IS NULL AND :P_VRORIGFIN IS NULL))
                    	AND ((ROUND(CR.VRATUAREC, 2) BETWEEN ROUND(:P_VRLIQUINI, 2) AND ROUND(:P_VRLIQUFIN, 2))
                    		 OR (:P_VRLIQUINI IS NULL AND :P_VRLIQUFIN IS NULL))
                    	AND ((TO_DATE(BR.DTLANCBAIXAR) BETWEEN :P_DTLANCBAIXARINI AND :P_DTLANCBAIXARFIN)
                    		 OR (:P_DTLANCBAIXARINI IS NULL AND :P_DTLANCBAIXARFIN IS NULL))";
            $cliente = " AND (NF.CDCLIENTE IN (";
        
            if(isset($params['P_CDCLIENTE'][0])) {
                $sqlCliente = $this->getSqlSelectNative($cliente, $params['P_CDCLIENTE']);
                $sql.=$sqlCliente;
                unset($params["P_CDCLIENTE"]);
            }
            $this->completeParamsArrayAccordingToQuery($sql, $params);
            if(array_key_exists("P_IDCONTABILIZA", $params)){
                if ($params["P_IDCONTABILIZA"] == 'S') {
                    $sql.= " AND (BR.NRLCTBBREC IN (SELECT NRLANCCONT
                            FROM LANCCONT LC
                            WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                                  LC.NRLANCCONT = BR.NRLCTBBREC))";
                } else if ($params["P_IDCONTABILIZA"] == 'N') {
                    $sql.= " AND (BR.NRLCTBBREC NOT IN (SELECT NRLANCCONT
                            FROM LANCCONT LC
                            WHERE LC.CDEMPRESA = NF.CDEMPRESA AND
                                  LC.NRLANCCONT = BR.NRLCTBBREC))";
                }
                unset($params["P_IDCONTABILIZA"]);
            }

        }
        $sql.= " ORDER BY BR.DTBAIXAREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    protected function getSqlSelectNative($prefix, $array) {
        foreach ($array as $value) {
                $cod = "'";
                $cod.=$value;
                $cod.="'";
                $prefix.=$cod.=",";
            }
        $size = strlen($prefix);
        $prefix = substr($prefix,0, $size-1);
        
        $sql = $prefix.="))";
        return $sql;
    }

    protected function completeParamsArrayAccordingToQuery($sql, &$params) {
        
        $matches = $this->getQueryParameters($sql);
        foreach($matches as $match) {
            if (!isset($params[$match]))
                $params[$match] = null;
        }
    }

    protected function getQueryParameters($sql) {
        $matches = [];
        //Encontra todos os parâmetros na query,
        preg_match_all('/\:(\w+)(?:\W|$)(?![\s\S]*\:\1(?:\W|$))/', $sql, $matches);

        return $matches[1] ? $matches[1] : array();
    }



    public function selectContopectbl($CDEMPRESA) {

        $params = array(
            'EMPRESA' => $CDEMPRESA,
            'NRORG' => $this->nrOrg
        );
        $sql = "SELECT
                CDEMPRESA,
                DTINICLANC,
                DTFINALANC,
                DTINICRELA,
                DTFINARELA,
                DTINICCTBL,
                DTFINACCTBL,
                IDCONTGERE,
                IDCONTCOME
                FROM
                CONTOPECTBL
                WHERE
                CDEMPRESA = :EMPRESA AND
                NRORG = :NRORG";

        $std = $this->connection->prepare($sql);

        $std->execute($params);

        return $std->fetch();

    }

}