<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\RetornoDDA;

class LeitorCnabBrasil{

    private $cnpjEmpresa;
    CONST ID_REGISTRO_LOTE = '1';
    CONST ID_REGISTRO_TRANSACAO = '3';

    CONST CODIGO_COMPENSACAO_BRASIL = '001';
    CONST CPF = '1';
    CONST SEGMENTO_G = 'G';

    protected $retornos = array();

    public function __construct($arquivo, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->existeBoleto = false;
        } catch (\Exception $e) {
            throw new \Exception('Operação Bloqueada! Ocorreu um erro ao executar a sua solicitação');
        }
    }

    public function getRetornos(){
        $boletos = array();
        $boletos = array_values(array_filter($this->retornos, function($item){
          return strtoupper($item->getTipoRegistro()) == self::SEGMENTO_G;
        }));
        return $boletos;
    }

    public function leArquivo($nrorg, $accesStage = false)
    {
        try {
            foreach ($this->arquivo as $linha) {
                if($linha){
                    if (strlen($linha) > 0){
                        $this->retorno = new Retorno();
                        $tipoDeRegistroDoArquivo = substr($linha,7,1);
                        switch ($tipoDeRegistroDoArquivo) {
                            // case(self::ID_REGISTRO_HEADER):
                            //     $this->leHeaderArquivo($linha);
                            //     break;
                            case(self::ID_REGISTRO_LOTE):
                                $this->leHeaderLote($linha);
                                break;
                            case(self::ID_REGISTRO_TRANSACAO):
                                $this->existeBoleto = 'S';
                                $this->leRegistroDetalhe($linha);
                                break;
                        }
                        $this->retornos[] = $this->retorno;
                    }
                }
            }
            if(!$this->existeBoleto){
                if(!$accesStage)
                     throw new \Exception("Não há boletos no arquivo de retorno.", 1);
                else return array('Error' => ('Não há boletos no arquivo de retorno.'));
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function leHeaderArquivo($registro){
        $this->cnpjEmpresa = substr($registro, 18, 14);
    }

    private function leHeaderLote($registro){
       
        // Número de Inscrição da Empresa
        // Posição 19 a 33
        // Picture: 9(15)
        $this->cnpjEmpresa = substr($registro, 18, 14);
    }

    private function leRegistroDetalhe($registro){
        
        // Código no Banco na compensação
        // Posição 1 a 3 
        // Picture: 9(3)
        $codigoBanco = substr($registro, 0, 3);
        
        
        // Cód. Segmento do Registro Detalhe
        // Posição 14 a 14 
        // Picture: X(1)
        if ((substr($registro, 13, 1) == self::SEGMENTO_G) && ($codigoBanco == self::CODIGO_COMPENSACAO_BRASIL)){
            $this->retorno->setTipoRegistro(self::SEGMENTO_G);
            
            // Valor Nominal do Título
            // Posição 116 a 130
            // Picture: 9(15)
            $valorTitulo = floatVal(substr($registro, 115, 15))/100;
            $this->retorno->setVrfinalbpag($valorTitulo); 
            
            // Data de Vencimento do Título
            // Posição 108 115
            // Picture: X(8) DD/MM/YYYY
            $dtvencimento = substr($registro, 107, 2) . '/' . substr($registro, 109, 2) . '/' . substr($registro, 111, 4);
            $this->retorno->setDtvenbolpag($dtvencimento);
            
            // Código de Barras
            // Posição 18 a 61
            // Picture: 9(44)
            $this->retorno->setCdbarrapag(substr($registro, 17, 44));
            
            // Tipo de Inscrição do Cedente
            // Posição 62 a 62
            // Picture: 9(1)
            //'0' = Isento / Não Informado *** '1' = CPF *** '2' = CGC / CNPJ *** '3' = PIS / PASEP *** '9' = Outros
            $codigoInscricaoCedente = substr($registro, 61, 1);
            $tipoInscricao = ($codigoInscricaoCedente == self::CPF) ? 'F' : 'J';
            $this->retorno->setTipoInscricao($tipoInscricao);
            
            // Número de Inscrição do Cedente
            // Posição 63 a 77
            // Picture: 9(15)
            $this->retorno->setCdforneced(substr($registro, 63, 14));
            
            // Nome do Cedente
            // Posição 78 a 107
            // Picture: X(30)
            $this->retorno->setNmforneced(substr($registro, 77, 30));
            
            // Número do Documento de Cobrança
            // Posição 148 a 162
            // Picture: 9(15)
            $this->retorno->setNrnotafisc(trim(substr($registro, 147, 15)));
            
            $this->retorno->setCnpjEmpresa($this->cnpjEmpresa);
            $this->retorno->setCodigoBanco($codigoBanco);
        }
    }

}