<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoBrasilService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoBtgService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 11;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';


    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        
        $nossoNumerocalc = substr($this->getCarteira(),0,2).$nossoNumero;
        
        return $nossoNumerocalc.$this->getDigitoVerificadorNossoNumero($nossoNumerocalc);
    }

/** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Itaú.
     * @param $nossoNumero
     * @return string
     */

    public function getLinhaDigitavel($nossoNumero)
    {
        /** NOTAS CAMPO 1  
         * Cod Banco - 3 Posições
         * Moeda - 1 Posição
         * Posições 20 a 24 no Código de Barras (Agencia 4 Dígitos + 1 Posição da Carteira)
         * DV do Campo calculado MOD 10 
        */
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $compCarteira = substr($this->boleto->getCdcartcobbol(), 0, 1);
        $grupo1  = $this->getIdentificadorBanco().self::MOEDA_REAL.$cdagencia.$compCarteira;
        $grupo1 .= $this->avaliarDacLinha($this->getVerificadorGruposLinhaDigitavel($grupo1));
        
        
        /** NOTAS CAMPO 2  
         * Posições 25 a 34 no Código de Barras (2º Posição da Carteira, 9 primeiras posições do Código Nosso Numero sem o DV e sem a Carteira)
         * DV do Campo 2 (calculado com o Módulo 10)
        */
        $compCarteiraSegundoGrupo = substr($this->boleto->getCdcartcobbol(), 1, 1);
        $compNossoNumero = substr($nossoNumero, 2, 9);
        $grupo2 = $compCarteiraSegundoGrupo.$compNossoNumero;
        $grupo2 .= $this->avaliarDacLinha($this->getVerificadorGruposLinhaDigitavel($grupo2));
        
        /** NOTAS CAMPO 3  
         * Posições 35 a 44 no Código de Barras (Duas ultimas posições do Codigo Nosso Numero Sem o DV + Conta Corrente sem o DV)
         * DV do Campo 3 (calculado com o Módulo 10)
        */
        $compNossoNumeroGrupo3 = substr($nossoNumero, 11,2);
        $cdcontcorr = str_pad(substr($this->getCdcontcorr(), 0, 5),7,0, STR_PAD_LEFT);
        $grupo3 = $compNossoNumeroGrupo3.$cdcontcorr.'0';
        $grupo3 .= $this->avaliarDacLinha($this->getVerificadorGruposLinhaDigitavel($grupo3));
        
        /** NOTAS CAMPO 4  
         * DV do Código de Barras
        */
        $grupo4 = $this->getDigitoCodigoBarras();
        
        /** NOTAS CAMPO 5  
         * Fator de Vencimento - 4 Posições
         * Valor do boleto (com 2 casas decimais sem ponto) - 10 posições
        */
        $grupo5 = $this->getFatorVencimento().$this->getValorNominal();
        return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        
        //Esta string será substituída pelo código do dígito verificador
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        
        $codigoDeBarras .= $this->getFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        
        //Campo Livre 
        $cdagencia = substr($this->getCdAgencia(), 0, 4); //Sem dígito
        $cdcartcobbol = str_pad(substr($this->boleto->getCdcartcobbol(),0,2), 2, 0, STR_PAD_LEFT);
        
        $nossoNumero = str_pad(substr($nossoNumero,2, 11),0, STR_PAD_LEFT); 
        $cdcontcorr = str_pad(substr($this->getCdcontcorr(), 0, 5),7,0, STR_PAD_LEFT);
        
        $codigoDeBarras .= $cdagencia;
        $codigoDeBarras .= $cdcartcobbol;
        $codigoDeBarras .= $nossoNumero;
        $codigoDeBarras .= $cdcontcorr;
        $codigoDeBarras .= '0'; //Zero Fixo conforme manual;
        
        $digito         = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);
        
        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }

    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        $digito = $total % 10;

        return 10 - $digito;
    }
    
    private function getDacAgenciaConta($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 7){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        $digito = $total % 10;
        
        $dac = 10 - $digito;
        
        if($dac === 10){
            return '0';
        } else {
            return $dac;
        }

    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

    private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

        $linhadigitavel = substr($grupo1,0,5) . '.' . substr($grupo1, 5) . ' ';

        $linhadigitavel .= substr($grupo2,0, 5) . '.' . substr($grupo2, 5) . ' ';

        $linhadigitavel .= substr($grupo3,0, 5) . '.' . substr($grupo3, 5) . ' ';

        $linhadigitavel .= $grupo4 . ' ';

        $linhadigitavel .= $grupo5;

        return $linhadigitavel;
    }

    private function getDigitoVerificadorNossoNumero($sequencia){
        // Inverte o nosso numero.
        $sequencia   = strrev($sequencia);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $result = ((int)substr($sequencia, $i, 1) * $peso);
            $acumulador += $result;
            $peso = $this->incrementaValorPeso($peso,2, 7);
        }
        
        $digito = $acumulador % 11;
        
        $digito = $this->avaliarDigitoRetornoNossoNumero($digito);

        $this->setDigitoCodigoBarras($digito);

        return $digito;
    }

    protected function avaliarDigitoRetornoNossoNumero($digito)
    {
        /*
         * * NOTAS MANUAL
         * SE o resto da divisão for "1" DV = "P"
         * SE o resto da divisão for "0" DV = "0"
         * SE não calcula modulo 11 - resto
         */
 
         if($digito == '1'){
             return 'P';
         } elseif($digito == '0'){
             return '0';
         } else {
             $result = self::MODULO_DIGITO_VERIFICADOR - $digito;
             return $result;
         }
    }
    
    protected function avaliarDacLinha($digito)
    {
        if ($digito == 10) return 1;
        else return $digito;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
}
?>