<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\ValidacaoEmissaoBoletoService;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;

/**
 * Class EmissaoBoletoService
 * @package Teknisa\FIN\Modules\ContaReceber\Controllers
 */
class EmissaoBoletoService{

    protected $entityManager;
    protected $titulosAReceberFactory;

    /**
     * @param EntityManager $entityManager
     * @param TitulosAReceberFactory $titulosAReceberFactory
     */
    public function __construct(EntityManager $entityManager, TitulosAReceberFactory $titulosAReceberFactory)
    {
        $this->entityManager = $entityManager;
        $this->titulosAReceberFactory = $titulosAReceberFactory;
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return null
     * @throws \Exception
     */
    public function emiteBoleto($nrorg, $cdfilial, $nrlanbolrec)
    {
        try {
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);
            
            // Executa validacoes quanto a emissao de boleto. Esta validacao pode variar de acordo com bancos distintos.
            $validacao = ValidacaoEmissaoBoletoService::factory($boleto)->valida();
            
            // Emissão do Boleto a Receber
            $emissaoBoleto = GeraEmissaoBoletoService::factory($boleto);
            
            //VerificaÃ§Ã£o se o boleto ja foi emitido.
            if ($boleto->getIdstatbrec() != 'E'){

                $nossoNumero    = $boleto->getCdnosnumrec()?str_replace([' ', '-'], '', $boleto->getCdnosnumrec()):$emissaoBoleto->getNossoNumeroComDigito();
                $codigoDeBarras = $emissaoBoleto->getCodigoDeBarras($nossoNumero);
                $linhaDigitavel = $emissaoBoleto->getLinhaDigitavel($nossoNumero);

                $boleto->setCdnosnumrec(substr($nossoNumero,0,-1).'-'.substr($nossoNumero,-1));
                $boleto->setDslinhabarrec($linhaDigitavel);
                $boleto->setCdbarrarec($codigoDeBarras);
                $boleto->setIdstatbrec('E');
                
                // Atualiza Boleto por Assinatura (SOMENTE P/ "NO CASH")
                $this->titulosAReceberFactory->atualizaCodigoBarraBoletrecassina($boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec(), $boleto->getIdstatbrec(), $boleto->getCdbarrarec(), $boleto->getDslinhabarrec());
                
                $this->entityManager->persist($boleto);
                $this->entityManager->flush();
            }

            return $emissaoBoleto->getRelatorioBoleto($boleto->getNrorg(),$boleto->getCdreducont());
        } catch (\Exception $e) {
            throw $e;
        }
    }
}