var Bco11100_lancamentoContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;

    self.forceActivate = function(){
        templateManager.container.getWidget('lancamentoContaCorrente').activate();
    };

    self.openFilter = function(lancamento){
        self.iniciaLancamentoFiltro(lancamento)
        .then(function(filterRow){
            self.buscaPadraoEmpresa(filterRow).then(function(filterRow){
                self.abrirFiltro(lancamento, filterRow);
            });

        });
    };
    
    self.buscaPadraoEmpresa = function(filterRow) {
        var deferred = ZHPromise.defer();
        var lancamentoContaCorrente = templateManager.container.getWidget('lancamentoContaCorrente');
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial)
        .then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                filterRow.CDEMPRESA = empresa.CDEMPRESA;
                filterRow.NMEMPRESA = empresa.NMEMPRESA;
                lancamentoContaCorrente.getField('DSCONTCORR').dataSourceFilter[0].value = cdempresa;
                deferred.resolve(filterRow);
            });
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    self.abrirFiltro = function(lancamento, rowFilter){
        var deferred = ZHPromise.defer();
        ZhFilterPreferences.setOriginalFilter(lancamento.id, rowFilter);
        ZhFilterPreferences.openFilterWidget(lancamento, rowFilter);
        deferred.resolve();
        return deferred.promise;
    };

    self.iniciaLancamentoFiltro = function(lancamento){
        var deferred = ZHPromise.defer();
        var route = 'fin#/crud/geral_bco';
        var rowFilter = {};
        //rowFilter.VRMOVI_INI = 0;
        //rowFilter.VRMOVI_FIN = 0;
        rowFilter.DEBCRED = 'T';
        General.callRestEngine(route, 'FilterData', [])
            .then(function(response){
                self.setDataLancamentoNoFiltro(lancamento, response, 'DTMOVICRED')
                .then(function(intervalo){
                    rowFilter.DTMOVICRED = intervalo;
                })
                .catch(function(){});
                lancamento.dataSourceFilter.forEach(function(filtro){
                    switch(filtro.name){
                        case 'VRTODOS': filtro.value = 'T';
                            break;
                        case 'DEBCRED': filtro.value = 'T';
                            break;
                        case 'TTIPODEBCRED': filtro.value = 'T';
                            break;
                    }
                });
                lancamento.getField('TIPODEBCRED').filterProperties.isVisible = true;
                deferred.resolve(rowFilter);
            })
            .catch(function(erro){
                deferred.resolve(rowFilter);
            });
        return deferred.promise;
    };

    self.setDataLancamentoNoFiltro = function(lancamento, response, data){
        var deferred = ZHPromise.defer();
        var dtmovimento = lancamento.dataSourceFilter.filter(function(filtro){
            return filtro.name == data;
        });
        if (dtmovimento.length == 0){
            var diasAnt  = response.dataset.geral_bco[0].QTDIAANTLANC;
            var diasPos  = response.dataset.geral_bco[0].QTDIAAPOLANC;
            var start = moment().subtract(diasAnt, 'days').format('DD/MM/YYYY');
            var end   = moment().add(diasPos, 'days').format('DD/MM/YYYY');
            var periodoInicial = {
                "name": data,
                "value": [start, end],
                "operator": "BETWEEN",
                "static": true
            };
            //lancamento.dataSourceFilter.push(periodoInicial);
            var intervalo = {start: start, end: end};
            deferred.resolve(intervalo);
            //lancamento.getField(data).filterProperties.newLine = true;
        }else{
            deferred.reject();
        }
        return deferred.promise;
    };

    /**
     * Prepara Filtro de Lançamento
     */
    self.preparaFiltroLancamento = function(args){
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        
        var filtroLancamento = args.e.preference;
        var valorInicial = filtroLancamento.FILTER_ROW.VRMOVI_INI;
        var valorFinal = filtroLancamento.FILTER_ROW.VRMOVI_FIN;
        var tipodebcred = filtroLancamento.FILTER_ROW.TIPODEBCRED;
        
        filtroLancamento.FILTER_ROW.TIPODEBCRED = Array.isArray(filtroLancamento.FILTER_ROW.TIPODEBCRED) ? filtroLancamento.FILTER_ROW.TIPODEBCRED.join('_') : filtroLancamento.FILTER_ROW.TIPODEBCRED;
        
        lancamento.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'VRMOVI_INI' || filtro.name == 'VRMOVI_FIN'){
                filtro.value = self.formataValoresDecimais(filtro.value);
            }
        });
        
        var fieldTipo = lancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.filterProperties.isVisible == true);
        });
        
        var tpdebcred_concat = '';
        
        tpdebcred_concat = filtroLancamento.FILTER_ROW.TIPODEBCRED ? null : 'T';
        
        // if (filtroLancamento.FILTER_ROW.DEBCRED == 'C')
        //     tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'C');
        // else if (filtroLancamento.FILTER_ROW.DEBCRED == 'D')
        //     tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'D');
        // else if (filtroLancamento.FILTER_ROW.DEBCRED == 'T')
        //     tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'T');
        
        lancamento.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'VRTODOS': filtro.value = ((valorInicial >= 0) && (valorFinal > 0) && (valorFinal >= valorInicial)) ? null : 'T';
                    break;
                case 'TIPODEBCRED': filtro.value = tpdebcred_concat;
                    break;
                case 'TTIPODEBCRED': filtro.value = filtroLancamento.FILTER_ROW.TIPODEBCRED ? null : 'T';
                    // if(tipodebcred === ''){ //verifica se o campo 'tipodebcred' esta vazio, se tiver ele vai passar T e se nao tiver, passa o valor preenchido
                    //     filtro.value = (tpdebcred_concat) ? null : 'T';
                    // }else{
                    //     filtro.value = tpdebcred_concat;
                    // }
                    break;
            }
        });
    };

    self.convertStringToArray = function(args){
        var filtroLancamento = args.e.preference;
        
        filtroLancamento.FILTER_ROW.TIPODEBCRED = Array.isArray(filtroLancamento.FILTER_ROW.TIPODEBCRED) ? filtroLancamento.FILTER_ROW.TIPODEBCRED.join('_') : filtroLancamento.FILTER_ROW.TIPODEBCRED.split('_');
    };

    /**
     * Prepara Filtro de Tipo de Lançamento 
     */
    self.preparaFiltroTipoLancamento = function(tipo, id){
        var tipo_checked     = tipo.dataSource.getCheckedRows();
        var primeiro         = true;
        var tpdebcred_concat = '';
        
        if (tipo_checked.length > 0){
            tipo_checked.forEach(function(row){
                if (id == 'C'){
                    if (primeiro){
                        tpdebcred_concat += id + row.CDTPCREDBANC;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + id + row.CDTPCREDBANC;
                }else if (id == 'D'){
                    if (primeiro){
                        tpdebcred_concat += id + row.CDTPDEBIBANC;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + id + row.CDTPDEBIBANC;
                }else if (id == 'T'){
                    if (primeiro){
                        tpdebcred_concat += row.IDTIPO + row.TIPODEBCRED;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + row.IDTIPO + row.TIPODEBCRED;
                }
            });
        }
        
        return tpdebcred_concat;
    };

    self.abrirPopupCheque = function (){
        var emissaoCheque = templateManager.container.getWidget('emissaoCheque');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        ScreenService.openPopup(emissaoCheque).then(
            function(){
                emissaoCheque.edit();
                emissaoCheque.getField('TIPOCHEQUE').setValue('A');
                emissaoCheque.getField('NRCHEQUE').dataSourceFilter[0].value = lancamento.currentRow.CDREDUCONT;
            }
        );
    };

    self.validaTipoCheque = function (widget){
        if (widget.currentRow.TIPOCHEQUE == 'A'){
            widget.getField('NOMINAL').readOnly = false;
        }else{
            widget.getField('NOMINAL').readOnly = true;
            widget.getField('NOMINAL').setValue('');
        }
    };

    self.confirmarEmissaoCheque = function(){
        var route = "fin_movimentacaoBancaria#/emissaoCheque";
        var emissaoCheque = templateManager.container.getWidget('emissaoCheque');
        if (emissaoCheque.isValid()){
            var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
            var nominal = (lancamento.currentRow.TIPOCHEQUE == 'A') ? emissaoCheque.currentRow.NOMINAL : null;
            var nrcheque = emissaoCheque.currentRow.NRCHEQUE;
            var row = {
                'VRCHEQUE': lancamento.currentRow.VRMOVICRED,
                'DTEMISCHEQ': lancamento.currentRow.DTMOVBANC,
                'NRMOVIDEBI': lancamento.currentRow.CODIGOLANCAMENTO,
                'CDREDUCONT': lancamento.currentRow.CDREDUCONT,
                'DSNOMCHEQ': nominal,
                'CDEMPRESA': lancamento.currentRow.CDEMPRESA,
                'IDCTBLBANC': lancamento.currentRow.IDCTBLBANC,
                'NRCHEQUE': emissaoCheque.currentRow.NRCHEQUE
            };
            General.callRestEngine(route, 'Row', row)
                .then(
                    function(){
                        self.setValoresDefaultCheque(lancamento, emissaoCheque).then(function(){
                            ScreenService.showMessage('Cheque salvo com sucesso.').then(function(){
                                if (emissaoCheque.currentRow.TIPOCHEQUE == "A"){
                                    ScreenService.confirmMessage('Cheque inicial '+lancamento.getField('NRCHEQUE').value()+'. Confirmar emissão do cheque?', 'question', function () {
                                        Bco11100_movimentacaoContaCorrente_Service.buscaMunicipio(getLocalVar('CDFILIAL'), lancamento.currentRow.NRORG).then(function (response) {
                                            var municipio = response.dataset.municipio_endefili[0].NMMUNICIPIO;
                                            self.getReport(municipio, lancamento.currentRow.CDREDUCONT, nrcheque);
                                        });
                                    });
                                }
                            });
                        });
                    }
                )
                .catch(function(erro){
                    if (contabilizacaoController.checkErrosContabilizacao(erro)){
                        ScreenService.showMessage(erro.data.error);
                    }else{
                        lancamento.activate();
                    }
                });
        }
    };

    self.setValoresDefaultCheque = function(lancamento, emissaoCheque){
        var deferred = ZHPromise.defer();
        ScreenService.closePopup();
        lancamento.activate();
        lancamento.getField('NRCHEQUE').isVisible = true;
        lancamento.getField('NOMINAL').isVisible = true;
        lancamento.getField('NRCHEQUE').setValue(emissaoCheque.getField('NRCHEQUE').value());
        lancamento.getField('NOMINAL').setValue(emissaoCheque.getField('NOMINAL').value());
        emissaoCheque.getField('NOMINAL').setValue('');
        emissaoCheque.getField('NRCHEQUE').setValue('');
        lancamento.getAction('EMITIRCHEQUE').isVisible = false;
        deferred.resolve();
        return deferred.promise;
    };

    self.getReport = function(municipio, cdreducont, nrcheque){
        var routes = 'fin_movimentacaoBancaria#/ModeloCheque';
        var row = {
            P_TESTE   : 'N',
            P_STLOCAL : municipio,
            P_CODREDU : cdreducont,
            P_NUMERO  : nrcheque
        };
        General.openReport(routes, 'ModeloCheque', row);
    };

    self.cancelLancamento = function (widget){
        General.widgetCancelForm(widget);
        widget.getField('NRCHEQUE').isVisible = true;
        widget.getField('NOMINAL').isVisible = true;
    };

    self.validaVisibilidadeCheque = function(widget){
        if (widget.currentRow.DEBCRED == 'D'){
            if (widget.currentRow.NRCHEQUE){
                widget.getField('NRCHEQUE').isVisible = true;
                widget.getField('NOMINAL').isVisible = true;
            }else{
                widget.getField('NRCHEQUE').isVisible = false;
                widget.getField('NOMINAL').isVisible = false;
            }
        }else{
            widget.getField('NRCHEQUE').isVisible = false;
            widget.getField('NOMINAL').isVisible = false;
        }
    };

    self.limitaValorDesconto = function(widget){
        var superiorDisponivel, superiorLancamento = false;
        var valorLancamento = parseFloat(templateManager.container.getWidget('lancamentoContaCorrente').currentRow.VRMOVICRED);
        var totalDesconto = 0.00;
        var totalDisponivel = 0.00;
        var indice = widget.currentRow.__index;
        valorDesconto = parseFloat(widget.currentRow.VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
        totalDisponivel = parseFloat(widget.currentRow.VRTOTDISP.toString().replace('.', '').replace(',', '.'));
        var data = widget.dataSource.data;
        for(var i = 0; i < data.length; i++){
            if (i != indice && data[i].__isSelected){
                totalDesconto += parseFloat(data[i].VRDESCONTO);
            }
        }
        if ((valorDesconto + totalDesconto) > totalDisponivel) {
            superiorDisponivel = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }
        if (valorDesconto > valorLancamento) {
            superiorLancamento = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }
        if (widget.currentRow.__isSelected) {
            desconto = parseFloat(totalDesconto) + parseFloat(widget.dataSource.data[indice].VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
            widget.getParent().currentRow.TOTALDESCONTO = desconto;
        }else{
            widget.getParent().currentRow.TOTALDESCONTO = parseFloat(totalDesconto);
        }
        if (superiorDisponivel){
            ScreenService.showMessage('Valor do desconto deve ser menor ou igual que o valor disponível.');
        }
        if (superiorLancamento){
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do lançamento.');
        }
    };

    self.setValorCliente = function(widget){
        if (widget.currentRow.CDFORNECED){
            widget.currentRow.CDCLIENTE = null;
            widget.currentRow.NRINSJURCLIE = null;
            widget.currentRow.NMRAZSOCCLIE = null;
        }
    };

    self.setValorFornecedor = function(widget){
        if (widget.currentRow.CDCLIENTE){
            widget.currentRow.CDFORNECED = null;
            widget.currentRow.NMRAZSOCFORN = null;
            widget.currentRow.NRINSJURFORN = null;
        }
    };

    self.setCurrentRowForm = function (widget, clifor){
        if (clifor == 'C') {
            widget.getParent().currentRow.NMRAZSOCCLIE = widget.getParent().getParent().currentRow.NMRAZSOCCLIE;
        }else{
            widget.getParent().currentRow.NMRAZSOCFORN = widget.getParent().getParent().currentRow.NMRAZSOCFORN;
        }
    };

    self.setDataSourcePopupCliente = function(popupCliente){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        popupCliente.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDCLIENTE') === 0){
                filtro.value = widget.currentRow.CDCLIENTE;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = widget.currentRow.CDEMPRESA;
            }
        });
        popupCliente.reload();
    };

    self.setDataSourcePopupFornecedor = function(popupFornecedor){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        popupFornecedor.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDFORNECED') === 0){
                filtro.value = widget.currentRow.CDFORNECED;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = widget.currentRow.CDEMPRESA;
            }else if (filtro.name.localeCompare('NRINSJURFORN') === 0) {
                filtro.value = widget.currentRow.NRINSJURFORN;
            }
        });
        popupFornecedor.reload();
    };

    self.newRowLancamento = function(widget){
        if(widget.appliedFilter){
            if(widget.appliedFilter.FILTER_ROW.CDREDUCONT){
                widget.getField('NRCHEQUE').isVisible = false;
                widget.getField('NOMINAL').isVisible = false;
                General.widgetNewRow(widget);
                widget.currentRow.CDREDUCONT = widget.appliedFilter.FILTER_ROW.CDREDUCONT;
                widget.currentRow.DSCONTCORR = widget.appliedFilter.FILTER_ROW.CONTACORRENTE;
                widget.currentRow.NMEMPRESA = widget.appliedFilter.FILTER_ROW.NMEMPRESA;
                widget.currentRow.CDEMPRESA = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
                widget.getField('NMFILIAL').dataSourceFilter[0].value = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
                widget.getField('NMFILIAL').reload(); 
            }else ScreenService.showMessage("Operação bloqueada. Selecione a Conta Corrente no filtro.");
        }else ScreenService.showMessage("Operação bloqueada. Selecione a Conta Corrente no filtro.");
    };

    self.saveLancamentoContaCorrente = function(lancamento){
        if (lancamento.isValid()) {
            if (parseFloat(lancamento.currentRow.VRMOVICRED) != 0.00) {
                if (lancamento.currentRow.__is_new) {
                    var routeAdiantamento = '';
                    var adiantamentoPopup = '';
                    if (lancamento.currentRow.CDCLIENTE) {
                        routeAdiantamento = 'fin_movimentacaoBancaria#/validaAdiantamentoCliente';
                        adiantamentoPopup = templateManager.container.getWidget('adiantamentoClienteTabbed');
                        msg = 'Atenção! Cliente possui crédito de adiantamento. Deseja utilizá-lo?';
                        self.openAdiantamento(lancamento, routeAdiantamento, adiantamentoPopup, msg, 'C');
                    } else if (lancamento.currentRow.CDFORNECED) {
                        routeAdiantamento = 'fin_movimentacaoBancaria#/validaAdiantamentoFornecedor';
                        adiantamentoPopup = templateManager.container.getWidget('adiantamentoFornecedorTabbed');
                        msg = 'Atenção! Fornecedor possui crédito de adiantamento. Deseja utilizá-lo?';
                        self.openAdiantamento(lancamento, routeAdiantamento, adiantamentoPopup, msg, 'F');
                    } else {
                        self.confirmarLancamentoContaCorrente(lancamento.currentRow.DEBCRED);
                    }
                } else {
                    self.confirmarLancamentoContaCorrente(lancamento.currentRow.DEBCRED);
                }
            }else{
                ScreenService.showMessage("Valor deve ser diferente de zero.");
            }
        }
    };

    self.openAdiantamento = function(lancamento, routeAdiantamento, adiantamentoPopup, msg, clifor){
        var msgDev, routeDevolucao, devolucaoPopup = '';
        if (clifor == 'C') {
            msgDev = 'Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';
            devolucaoPopup = templateManager.container.getWidget('devolucaoClienteTabbed');
        }else{
            msgDev = 'Atenção! Fornecedor possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';
            devolucaoPopup = templateManager.container.getWidget('devolucaoFornecedorTabbed');
        }
        General.callRestEngine(routeAdiantamento, 'Row', lancamento.currentRow)
            .then(function(adiantamento) {
                if (adiantamento.dataset.ADIANTAMENTO.length > 0) {
                    ScreenService.confirmMessage(msg, 'question',
                        function() {
                            ScreenService.openPopup(adiantamentoPopup).then(function () {
                                self.setCurrentRowForm(adiantamentoPopup.widgets[0], clifor);
                                adiantamentoPopup.widgets[0].reload();
                            });
                        },
                        function(){
                            self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
                        });
                }else{
                    self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
                }
            })
            .catch(function(erro){
                self.confirmarLancamentoContaCorrente(clifor);
            });
    };

    self.openDevolucao = function(lancamento, routeDevolucao, devolucaoPopup, msg, clifor){
        // Flag para Devolução utilizada em Lançamento de Conta Corrente
        lancamento.currentRow.LANCAMENTOCC = 'S';
        
        General.callRestEngine(routeDevolucao, 'Row', lancamento.currentRow)
            .then(function(devolucao) {
                if (devolucao.dataset.DEVOLUCAO.length > 0) {
                    ScreenService.confirmMessage(msg, 'question', function(success){
                        ScreenService.openPopup(devolucaoPopup).then(function(){
                            self.setCurrentRowForm(devolucaoPopup.widgets[0], clifor);
                            devolucaoPopup.widgets[0].reload();
                        });
                    });
                }else{
                    self.confirmarLancamentoContaCorrente(clifor);
                }
            })
            .catch(function(erro){
                self.confirmarLancamentoContaCorrente(clifor);
            });
    };

    self.confirmarAdiantamento = function(widget, clifor){
        var msgDev, msgConfirme, routeDevolucao, devolucaoPopup = '';
        if (clifor == 'C') {
            msgDev = 'Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';
            devolucaoPopup = templateManager.container.getWidget('devolucaoClienteTabbed');
        }else{
            msgDev = 'Atenção! Fornecedor possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';
            devolucaoPopup = templateManager.container.getWidget('devolucaoFornecedorTabbed');
        }
        msgConfirme = 'É necessário selecionar os registros para confirmação do Adiantamento.';
        if (self.validacaoConfirmePopup(widget.widgets[0])) {
            lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
            self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
        }else{
            ScreenService.showMessage(msgConfirme);
        }
    };

    self.confirmarDevolucao = function(devolucao, clifor) {
        msgConfirme = 'É necessário selecionar os registros para confirmação da Devolução.';
        if (self.validacaoConfirmePopup(devolucao.widgets[0])) {
            self.confirmarLancamentoContaCorrente(clifor);
        } else {
            ScreenService.showMessage(msgConfirme);
        }
    };

    self.cancelarAdiantamento = function(routeDevolucao, clifor, widget){
        var devolucaoPopup = (clifor == 'C') ? templateManager.container.getWidget('devolucaoClienteTabbed') : templateManager.container.getWidget('devolucaoFornecedorTabbed');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
        General.callRestEngine(routeDevolucao, 'Row', lancamento.currentRow)
            .then(function(devolucao) {
                if (devolucao.length > 0) {
                    ScreenService.confirmMessage('Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function(success){
                        ScreenService.openPopup(devolucaoPopup).then(function(){
                            self.setCurrentRowForm(devolucaoPopup.widgets[0], clifor);
                        });
                    });
                }else{
                    self.confirmarLancamentoContaCorrente(clifor);
                }
            });
    };

    self.cancelarDevolucao = function(clifor, widget){
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
        self.confirmarLancamentoContaCorrente(clifor);
    };

    /**
     * Ao confirmar um novo lancamento em Conta Corrente são
     * realizadas validações em uma popup de adiantamento ou
     * desconto para clientes e fornecedores. Após validação
     * a ação de lançamento é continuada.
     */
    self.confirmarLancamentoContaCorrente = function(clifor){
        var adiantamento, devolucao = '';
        var routeSave = 'fin_movimentacaoBancaria#/saveLancamentoContaCorrente';
        //ScreenService.closePopup();
        //ScreenService.openWindow('fin_movimentacaoBancaria#/bco11100_movimentacaoContaCorrente');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        if (clifor == 'C') {
            adiantamento = templateManager.container.getWidget('adiantamentoCliente');
            devolucao = templateManager.container.getWidget('devolucaoCliente');
        }else{
            adiantamento = templateManager.container.getWidget('adiantamentoFornecedor');
            devolucao = templateManager.container.getWidget('devolucaoFornecedor');
        }
        var dataSet = [];
        lancamento.currentRow.CDCLIENTE = (lancamento.currentRow.hasOwnProperty('CDCLIENTE')) ? lancamento.currentRow.CDCLIENTE : '';
        lancamento.currentRow.CDFORNECED = (lancamento.currentRow.hasOwnProperty('CDFORNECED')) ? lancamento.currentRow.CDFORNECED : '';
        dataSet[0] = lancamento.currentRow;
        var data = (adiantamento.dataSource.data) ? adiantamento.dataSource.data : [];
        for(var i = 0; i < data.length; i++){
            if (data[i].__isSelected){
                dataSet[dataSet.length] = data[i];
            }
        }
        var dataDev = (devolucao.dataSource.data) ? devolucao.dataSource.data : [];
        for(i = 0; i < dataDev.length; i++){
            if (dataDev[i].__isSelected){
                dataSet[data.length + dataSet.length] = dataDev[i];
            }
        }
        dataSet[0].QTDEADIANTAMENTO = adiantamento.getCheckedRows().length;
        
        localStorage.setItem('lancBancRow',JSON.stringify(dataSet));
        localStorage.setItem('TIPOBAIXA','LANCBANC');
        
        General.callRestEngine(routeSave, 'DataSet', dataSet)
            .then(function (response) {
                ScreenService.closeSwipe().then(function(){
                    lancamento.reload();
                    ScreenService.showMessage('Registro salvo com sucesso');
                    ScreenService.closePopup();
                    sessionStorage.setItem('ERROCONTAB', false);
                });
                lancamento.activate();
                lancamento.currentRow.__is_new = false;
            })
            .catch(function(erro){
                if (contabilizacaoController.checkErrosContabilizacao(erro)){
                    ScreenService.showMessage(erro.data.error);
                }else{
                    lancamento.activate();
                }
            });
    };

    //Calcula o total disponível de contas do tipo adiantamento para cliente ou fornecedor
    self.calculaTotalAdiantamentoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += data.VRTOTDISP;
        });
        widget.getParent().currentRow.TOTALADIANTAMENTO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    //Calcula o total disponível de contas do tipo devolucao para cliente ou fornecedor
    self.calculaTotalDevolucaoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += data.VRDISPONIVEL;
        });
        widget.getParent().currentRow.TOTALDEVOLUCAO = totalDisponivel;
    };

    self.selecionaTodosRegistros = function(widget){
        self.atualizaValorAdiantaDev(widget).then(
            function() {},
            function(error){
                widget.unSelectAll();
            }
        );
    };

    self.atualizaValorAdiantaDev = function(widget) {
        var deferred = ZHPromise.defer();
        var totalDesconto = 0;
        widget.getCheckedRows().forEach(function (data) {
            if (data.__isSelected) {
                totalDesconto += parseFloat(data.VRDESCONTO);
            }
        });
        var valorLancamento = parseFloat(templateManager.container.getWidget('lancamentoContaCorrente').currentRow.VRMOVICRED);
        if (totalDesconto > valorLancamento){
            ScreenService.showMessage('O valor a descontar não pode ser maior que o valor do lançamento.');
            widget.getLastSelectedRow().__isSelected = false;
            deferred.reject();
        }else {
            deferred.resolve();
            widget.getParent().currentRow.TOTALDESCONTO = totalDesconto;
        }
        return deferred.promise;
    };

    self.validacaoConfirmePopup = function(widget){
        var checked = false;
        var data = widget.dataSource.data;
        for (var i = 0; i < data.length && !checked; i++) {
            if (data[i].__isSelected){
                checked = true;
            }
        }
        return checked;
    };

    self.setTipoLancamento = function(widget){
        if (widget.currentRow.DEBCRED == 'C'){
            widget.currentRow.TIPOCREDITO = widget.currentRow.TIPO;
            self.validaLancamento(widget);
            widget.getAction('EMITIRCHEQUE').isVisible = false;
        }
        else{
            widget.currentRow.TIPODEBITO = widget.currentRow.TIPO;
            self.validaLancamento(widget);
            if (widget.currentRow.NRCHEQUE)
                 widget.getAction('EMITIRCHEQUE').isVisible = false;
            else widget.getAction('EMITIRCHEQUE').isVisible = true;
        }
    };

    self.deleteLancamentoContaCorrente = function(lancamento){
        var route = 'fin_movimentacaoBancaria#/deleteLancamentoContaCorrente';
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question',
            function (success) {
                General.callRestEngine(route, 'Row', lancamento.currentRow).then(function (mensagem) {
                    ScreenService.closeSwipe().then(function () {
                        lancamento.activate();
                        lancamento.reload();
                        ScreenService.showMessage(mensagem.messages[0].message);
                    });
                }).catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }
        );

    };

    self.validaTipoLancamento = function(widget){
        var tipocredito = widget.getField('TIPOCREDITO');
        var tipodebito = widget.getField('TIPODEBITO');
        var tipodebcred = widget.getField('TIPODEBCRED');
        if (widget.getField('DEBCRED').value() == 'C') {
            tipodebito.setValue('');
            tipodebito.isVisible = false;
            tipodebito.showOnForm = false;
            tipodebito.showOnView = false;
            tipodebito.class = "0";
            tipocredito.isVisible = true;
            tipocredito.showOnForm = true;
            tipocredito.showOnView = true;
            tipocredito.class = "6";
        }else if (widget.getField('DEBCRED').value() == 'D'){
            tipocredito.setValue('');
            tipocredito.isVisible = false;
            tipocredito.showOnForm = false;
            tipocredito.showOnView = false;
            tipocredito.class = "0";
            tipodebito.isVisible = true;
            tipodebito.showOnForm = true;
            tipodebito.showOnView = true;
            tipodebito.class = "6";
        }
        widget.getField('DESCRICAO').setValue('');
        tipodebito.reload();
        tipocredito.reload();
    };

    self.validaTipoLancamentoFiltro = function(filtroLancamento){
        var tipoCred = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED2' && filtro.descriptionField == 'DSTPCREDBANC');
        })[0];
        var tipoDeb = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED1' && filtro.descriptionField == 'DSTPDEBIBANC');
        })[0];
        var tipoDebCred = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.descriptionField == 'DESCRICAO');
        })[0];
        if (filtroLancamento.getField('DEBCRED').value() == 'C') {
            tipoDeb.setValue('');
            tipoDebCred.setValue('');
            tipoDeb.isVisible = false;
            tipoDeb.showOnForm = false;
            tipoDeb.showOnView = false;
            tipoDeb.class = "0";
            tipoCred.isVisible = true;
            tipoCred.showOnForm = true;
            tipoCred.showOnView = true;
            tipoCred.class = "6";
            tipoDebCred.isVisible = false;
            tipoDebCred.showOnForm = false;
            tipoDebCred.showOnView = false;
            tipoDebCred.class = "0";
        }else if (filtroLancamento.getField('DEBCRED').value() == 'D') {
            tipoCred.setValue('');
            tipoDebCred.setValue('');
            tipoDeb.isVisible = true;
            tipoDeb.showOnForm = true;
            tipoDeb.showOnView = true;
            tipoDeb.class = "6";
            tipoCred.isVisible = false;
            tipoCred.showOnForm = false;
            tipoCred.showOnView = false;
            tipoCred.class = "0";
            tipoDebCred.isVisible = false;
            tipoDebCred.showOnForm = false;
            tipoDebCred.showOnView = false;
            tipoDebCred.class = "0";
        }else if (filtroLancamento.getField('DEBCRED').value() == 'T') {
            tipoCred.setValue('');
            tipoCred.setValue('');
            tipoDeb.isVisible = false;
            tipoDeb.showOnForm = false;
            tipoDeb.showOnView = false;
            tipoDeb.class = "0";
            tipoCred.isVisible = false;
            tipoCred.showOnForm = false;
            tipoCred.showOnView = false;
            tipoCred.class = "0";
            tipoDebCred.isVisible = true;
            tipoDebCred.showOnForm = true;
            tipoDebCred.showOnView = true;
            tipoDebCred.class = "6";
        }
        tipoDeb.reload();
        tipoCred.reload();
        tipoDebCred.reload();
    };

    self.validaLancamento = function(widget){
        var tipocredito = widget.getField('TIPOCREDITO');
        var tipodebito = widget.getField('TIPODEBITO');
        if (widget.getField('DEBCRED').value() == 'C') {
            tipodebito.isVisible = false;
            tipodebito.showOnForm = false;
            tipodebito.showOnView = false;
            tipodebito.class = "0";
            tipocredito.isVisible = true;
            tipocredito.showOnForm = true;
            tipocredito.showOnView = true;
            tipocredito.class = "6";
        }else{
            tipocredito.isVisible = false;
            tipocredito.showOnForm = false;
            tipocredito.showOnView = false;
            tipocredito.class = "0";
            tipodebito.isVisible = true;
            tipodebito.showOnForm = true;
            tipodebito.showOnView = true;
            tipodebito.class = "6";
        }
    };

    self.setTipoFiltro = function(widget){
        var tipoLancamento = widget.getField('DEBCRED');
        widget.getField('TIPOFILTRO').dataSourceFilter[0].value = tipoLancamento.value();
    };

    self.atualizaValoresSelectMultiple = function(widget){
        if (widget.getField('TIPO').filterProperties.copyValueSelectMultiple) {
            var valorConcatenado = widget.appliedFilter.FILTER_ROW.TIPO;
            widget.getField('TIPO').setValue(widget.getField('TIPO').filterProperties.copyValueSelectMultiple);
            widget.appliedFilter.FILTER_ROW.TIPO = valorConcatenado;
        }
    };

    self.setDescricaoLancamento = function(widget){
        widget.currentRow.DESCRICAO = (widget.currentRow.DEBCRED == 'C')? widget.currentRow.TIPOCREDITO : widget.currentRow.TIPODEBITO;
        widget.currentRow.TIPO = widget.currentRow.DESCRICAO;
    };

    self.setDefaultValue = function(fields){
        fields.forEach(function(field){
            if (field.value().length == 0){
                field.setValue("0,00");
            }
        });
    };

    self.validaActionContabilizacaoLancamento = function(widget){
        if (widget.currentRow.NRLANCCONT)
             widget.getAction('CONTABILIZACAO').isVisible = true;
        else widget.getAction('CONTABILIZACAO').isVisible = false;
    };

    self.consultarContabilizacaoLancamento = function(){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        if (widget.currentRow.NRLANCCONT){
            var contabil = [];
            contabil.push({});
            contabil[0].CDEMPRESA  = widget.currentRow.CDEMPRESA;
            contabil[0].NRLANCCONT = widget.currentRow.NRLANCCONT;
            contabilizacaoController.exibeContabilizacao(contabil);
        }
    };
    
    /**
     * Formata Valores Decimais
     */
    self.formataValoresDecimais = function(valor){
        var valorFormatado = 
            (valor.toString().search(',') != '-1') ?
                ((parseFloat(valor.toString().split(',')[0]) > 0) ?
                    parseFloat(valor.toString().replace('.', '').replace(',', '.')) 
                 : parseFloat(valor.toString().replace(',', '.')))
            : parseFloat(valor);
            
        return valorFormatado;
    };
    
    self.habilitaContaCorrente = function(filtro){
        var lancamentoContaCorrente = templateManager.container.getWidget('lancamentoContaCorrente');
        
        filtro.getField('CONTACORRENTE').setValue('');
        
        if (filtro.currentRow.NMEMPRESA) {
            //lancamentoContaCorrente.getField('ORIGEM').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('CONTACORRENTE').readOnly = false;
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
        }else {
            filtro.getField('CONTACORRENTE').readOnly = true;
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = null;
        }
    };
    
    self.verificaRecarregamentoWidget = function(widget){
        if(sessionStorage.getItem('ERROCONTAB') === 'true')
            widget.reload();
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_lancamentoContaCorrente_Controller', Bco11100_lancamentoContaCorrente_Controller);
});